/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIURI;

/**
 * The nsIDOMPopupBlockedEvent interface is the datatype for events
 * posted when a popup window is blocked.
 */

[scriptable, builtinclass, uuid(f6e77f79-d178-419d-8177-dfae4392d318)]
interface nsIDOMPopupBlockedEvent : nsIDOMEvent
{
  /**
   * The window object that attempted to open the blocked popup
   * (i.e. the window object on which open() was called).
   */
  readonly attribute nsIDOMWindow requestingWindow;


  /**
   * The URI of the window that was blocked.
   */
  readonly attribute nsIURI popupWindowURI;

  /**
   * The window name passed to the window.open() call
   * (as the second argument)
   */
  readonly attribute DOMString popupWindowName;

  /**
   * The string of features passed to the window.open() call
   * (as the third argument)
   */
  readonly attribute DOMString popupWindowFeatures;

  void initPopupBlockedEvent(in DOMString typeArg,
                             in boolean canBubbleArg,
                             in boolean cancelableArg,
                             in nsIDOMWindow requestingWindow,
                             in nsIURI popupWindowURI,
                             in DOMString popupWindowName,
                             in DOMString popupWindowFeatures);
};

dictionary PopupBlockedEventInit : EventInit
{
  nsIDOMWindow requestingWindow;
  nsIURI popupWindowURI;
  DOMString popupWindowFeatures;
  DOMString popupWindowName;
};
